import sqlite3
import boto3
from dataclasses import dataclass
import pandas as pd
from botocore.exceptions import ClientError
import os
from typing import Optional

global REGION_NAME, S3_FILENAME, BUCKETNAME, DB_PATH
REGION_NAME: str = "us-east-1"
S3_FILENAME: str = "archivo.csv"
BUCKET_NAME: str = "COREANDSTRACT"
DB_PATH: str = "demo.db"


def obtener_ultima_fila_sqlite(db_path: str, tabla: str) -> pd.DataFrame:
    conn = sqlite3.connect(db_path)
    query = f"SELECT * FROM {tabla} ORDER BY id DESC LIMIT 1"  
    df = pd.read_sql_query(query, conn)
    conn.close()
    return df

def descargar_csv_de_s3(client, bucket_name: str, s3_filename: str, local_path: str, headers=None) -> bool:
    try:
        client.download_file(bucket_name, s3_filename, local_path)
        print(f"Descargado: s3://{bucket_name}/{s3_filename}")
        return True
    except ClientError as e:
        if e.response['Error']['Code'] == "404":
            print(f"Archivo no existe en S3. Se creará uno nuevo en {local_path}")
            if headers is not None and len(headers) > 0:
                df_vacio = pd.DataFrame(columns=headers)
                df_vacio.to_csv(local_path, index=False)
            return False
        else:
            raise

def agregar_fila_al_csv(nueva_fila: pd.DataFrame, local_path: str):
    df_csv = pd.read_csv(local_path)
    df_actualizado = pd.concat([df_csv, nueva_fila], ignore_index=True)
    df_actualizado.to_csv(local_path, index=False)

def subir_csv_a_s3(client, bucket_name: str, s3_filename: str, local_path: str):
    client.upload_file(local_path, bucket_name, s3_filename)
    print(f"Subido a s3://{bucket_name}/{s3_filename}")

@dataclass()
class MainDTO:
    aws_access_key_id: str
    aws_secret_access_key: str
    bucket_name: Optional[str]
    region_name: Optional[str]
    s3_filename: Optional[str]
    db_path: Optional[str]

def actualizar_csv_s3_con_sqlite(dto: MainDTO) -> None:
    
    local_csv_path = "/tmp/tmp.csv"
    db_path = dto.db_path or DB_PATH
    bucket_name = dto.bucket_name or BUCKET_NAME
    s3_filename = dto.s3_filename or S3_FILENAME

    client = boto3.client(
        's3',
        aws_access_key_id = dto.aws_access_key_id,
        aws_secret_access_key = dto.aws_secret_access_key,
        region_name = dto.region_name or REGION_NAME
    )

    nueva_fila = obtener_ultima_fila_sqlite(
        db_path=db_path, 
        tabla="cotizacion"
    )

    descargar_csv_de_s3(
        client=client, 
        bucket_name=bucket_name, 
        s3_filename=s3_filename, 
        local_path=local_csv_path, 
        headers=nueva_fila.columns
    )

    agregar_fila_al_csv(nueva_fila, local_csv_path)

  
    subir_csv_a_s3(client, bucket_name, s3_filename, local_csv_path)

## Develop
if __name__ == "__main__":
    import os
    from dotenv import load_dotenv

    load_dotenv()  

    dto = MainDTO(
        aws_access_key_id=os.getenv("AWS_ACCESS_KEY_ID"),
        aws_secret_access_key=os.getenv("AWS_SECRET_ACCESS_KEY"),
        bucket_name=None, #os.getenv("BUCKET_NAME")
        region_name=None, #os.getenv("AWS_REGION")
        s3_filename=None, #os.getenv("S3_FILENAME")
        db_path=os.getenv("DB_PATH")   
    )

    if os.getenv("AWS_BUCKET_NAME"):
        dto.bucket_name = os.getenv("AWS_BUCKET_NAME")

    if os.getenv("AWS_REGION"):
        dto.bucket_name = os.getenv("AWS_REGION")

    #dto.aws_access_key_id = ""
    #dto.aws_secret_access_key = ""

    actualizar_csv_s3_con_sqlite(dto)


import requests
from dataclasses import dataclass
from typing import Any
import sqlite3

global GLOBAL_URL, DB_PATH
GLOBAL_URL: str = "https://api.bluelytics.com.ar/v2/latest"
DB_PATH: str = "demo.db"

@dataclass(frozen=True, slots=True)
class ValueDTO:
    value_avg: float
    value_sell: float
    value_buy: float

@dataclass(frozen=True, slots=True)
class ResponseDTO:
    oficial: ValueDTO
    blue: ValueDTO
    oficial_euro: ValueDTO
    blue_euro: ValueDTO
    last_update: str

def clean_data(data: Any) -> ResponseDTO:

    return ResponseDTO(
        oficial=ValueDTO(
            value_avg=data["oficial"]["value_avg"],
            value_sell=data["oficial"]["value_sell"],
            value_buy=data["oficial"]["value_buy"]
        ),
        blue=ValueDTO(
            value_avg=data["blue"]["value_avg"],
            value_sell=data["blue"]["value_sell"],
            value_buy=data["blue"]["value_buy"]
        ),
        oficial_euro=ValueDTO(
            value_avg=data["oficial_euro"]["value_avg"],
            value_sell=data["oficial_euro"]["value_sell"],
            value_buy=data["oficial_euro"]["value_buy"]
        ),
        blue_euro=ValueDTO(
            value_avg=data["blue_euro"]["value_avg"],
            value_sell=data["blue_euro"]["value_sell"],
            value_buy=data["blue_euro"]["value_buy"]
        ),
        last_update=data["last_update"]
    )    

def save_data(data: ResponseDTO, db_path:str):
    conn = sqlite3.connect(db_path)

    cursor = conn.cursor()
    cursor.execute("CREATE TABLE IF NOT EXISTS cotizacion (id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT, value_avg REAL, value_sell REAL, value_buy REAL, last_update TEXT)")

    rows: list[tuple[str, float, float, float, str]] = [
        ("oficial", data.oficial.value_avg, data.oficial.value_sell, data.oficial.value_buy, data.last_update),
        ("blue", data.blue.value_avg, data.blue.value_sell, data.blue.value_buy, data.last_update),
        ("oficial_euro", data.oficial_euro.value_avg, data.oficial_euro.value_sell, data.oficial_euro.value_buy, data.last_update),
        ("blue_euro", data.blue_euro.value_avg, data.blue_euro.value_sell, data.blue_euro.value_buy, data.last_update)
    ]

    for row in rows:
        cursor.execute("INSERT INTO cotizacion (name, value_avg, value_sell, value_buy, last_update) VALUES (?, ?, ?, ?, ?)", row)

    conn.commit()
    conn.close()


def fetch_data(url: str) -> ResponseDTO:
    response = requests.get(url)

    if response.status_code == 200:
        raw_data = response.json()
        return clean_data(raw_data)
    else:
        raise Exception("Error al consultar la api")

def extract_and_load(db_path: str):

    data: ResponseDTO = fetch_data(GLOBAL_URL)

    save_data(data, db_path)

## Develop
if __name__ == "__main__":
    import os
    from dotenv import load_dotenv

    load_dotenv()  

    DB_PATH = os.getenv('DB_PATH')
    extract_and_load(DB_PATH)

    print("programa finalizado")





from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='core_extract_and_load',
    version='0.1.4',
    packages=find_packages(),
    description='Biblioteca para extraer datos via API y cargarlos',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='facundo vega',
    author_email='facundo.vega1234@gmail.com',
    zip_safe=False,
    url='https://github.com/facuvegaingenieer/example-core', 
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    install_requires=[
        'requests>=2.25.1',
    ],
    extras_require={
        'dev': [
            'pytest>=6.0',
            'pytest-cov>=2.0',
            'black>=21.0',
            'flake8>=3.8',
        ],
    },
)


from .main import extract_and_load, ValueDTO, ResponseDTO, fetch_data, clean_data, save_data
from .cleaning_and_saving_s3 import actualizar_csv_s3_con_sqlite, MainDTO
__all__ = [
    'extract_and_load',
    'ValueDTO', 
    'ResponseDTO',
    'fetch_data',
    'clean_data',
    'save_data',
    "actualizar_csv_s3_con_sqlite",
    "MainDTO"
]

__version__ = '0.1.4'